@@ -2,7 +2,7 @@
  * Gnome Chemistry Utils
  * object.cc 
  *
- * Copyright (C) 2002-2004 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -316,8 +316,15 @@ bool Object::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
 	return (m_Parent)? m_Parent->BuildContextualMenu (UIManager, object): false;
 }
 
-void Object::Add(GtkWidget* w)
+void Object::Add (GtkWidget* w)
 {
+	map<string, Object*>::iterator i;
+	Object* p = GetFirstChild (i);
+	while (p)
+	{
+		p->Add (w);
+		p = GetNextChild (i);
+	}
 }
 
 void Object::Print(GnomePrintContext *pc)
