@@ -4,7 +4,7 @@
  * Gnome Crystal
  * application.cc 
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -36,31 +36,12 @@
 #include <gcu/filechooser.h>
 #include "prefs.h"
 #include "window.h"
+#include <glib/gi18n.h>
 
 static unsigned short nNewDocs = 1;
 
-gcApplication::gcApplication(): Application ("gcrystal-unstable", DATADIR)
+gcApplication::gcApplication(): Application ("gcrystal-unstable")
 {
-	// get default programs
-	GConfClient* cli = gconf_client_get_default ();
-	if (cli) {
-		const char *value;
-		GConfEntry* entry = gconf_client_get_entry (cli, "/desktop/gnome/applications/browser/exec", NULL, true, NULL);
-		if (entry) {
-			value = gconf_value_get_string (gconf_entry_get_value (entry));
-			if (value) WebBrowser = value;
-		}
-		entry = gconf_client_get_entry (cli, "/desktop/gnome/url-handlers/mailto/command", NULL, true, NULL);
-		if (entry) {
-			value = gconf_value_get_string (gconf_entry_get_value (entry));
-			if (value) {
-				MailAgent = value;
-				int i = MailAgent.find (" %s");
-				if (i)
-					MailAgent.erase (i, MailAgent.size ());
-			}
-		}
-	}
 }
 
 gcApplication::~gcApplication ()
@@ -80,7 +61,7 @@ gcDocument *gcApplication::OnFileNew ()
 
 void gcApplication::OnFileOpen ()
 {
-	list<char const*> l;
+	list<string> l;
 	l.push_front ("application/x-gcrystal");
 	FileChooser (this, false, l);
 }
@@ -97,7 +78,7 @@ void gcApplication::OnFileSave ()
 
 void gcApplication::OnFileSaveAs ()
 {
-	list<char const*> l;
+	list<string> l;
 	l.push_front ("application/x-gcrystal");
 	FileChooser (this, true, l, m_pActiveDoc);
 }
@@ -152,31 +133,16 @@ void gcApplication::OnFilePrint ()
 	gnome_print_config_unref (config);
 }
 
-void gcApplication::OnExportJPEG ()
+void gcApplication::OnSaveAsImage ()
 {
 	if (!m_pActiveDoc)
 		return;
-	list<char const*> l;
-	l.push_front ("image/jpeg");
-	FileChooser (this, true, l, m_pActiveDoc, _("Export model as a Jpeg file..."));
-}
-
-void gcApplication::OnExportPNG ()
-{
-	if (!m_pActiveDoc)
-		return;
-	list<char const*> l;
-	l.push_front ("image/png");
-	FileChooser (this, true, l, m_pActiveDoc, _("Export model as a PNG file..."));
-}
-
-void gcApplication::OnExportVRML ()
-{
-	if (!m_pActiveDoc)
-		return;
-	list<char const*> l;
-	l.push_front ("image/vrml");
-	FileChooser (this, true, l, m_pActiveDoc, _("Export model as a VRML file..."));
+	list<string> l;
+	map<string, GdkPixbufFormat*>::iterator i, end = m_SupportedPixbufFormats.end ();
+	for (i = m_SupportedPixbufFormats.begin (); i != end; i++)
+		l.push_front ((*i).first.c_str ());
+	l.push_front ("model/vrml");
+	FileChooser (this, true, l, m_pActiveDoc, _("Save as image"), GetImageSizeWidget ());
 }
 
 gcDocument* gcApplication::GetDoc (const char* filename)
@@ -194,7 +160,7 @@ gcDocument* gcApplication::GetDoc (const char* filename)
 		return pDoc;
 	if (m_bFileOpening) {
 		pDoc = m_Docs.back ();
-		if (!pDoc->IsEmpty () || pDoc->IsDirty ())
+		if (!pDoc->GetEmpty () || pDoc->GetDirty ())
 			pDoc = NULL;
 	}
 	if (!pDoc) {
@@ -208,8 +174,7 @@ gcDocument* gcApplication::GetDoc (const char* filename)
 enum {
 	GCRYSTAL,
 	VRML,
-	JPEG,
-	PNG
+	PIXBUF
 };
 
 bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc)
@@ -217,15 +182,15 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 	gcDocument *Doc = static_cast<gcDocument*> (pDoc);
 	if (!mime_type)
 		mime_type = "application/x-gcrystal";
+	string filename2 = filename;
 	if (bSave) {
 		int type = GCRYSTAL;
-		if (!strcmp (mime_type, "image/png"))
-			type = PNG;
-		else if (!strcmp (mime_type, "image/jpeg"))
-			type = JPEG;
-		else if (!strcmp (mime_type, "image/vrml"))
+		char const *pixbuf_type = NULL;
+		if (!strcmp (mime_type, "model/vrml"))
 			type = VRML;
-		char *filename2, *ext = "";
+		else if ((pixbuf_type = GetPixbufTypeName (filename2, mime_type)))
+			type = PIXBUF;
+		char *ext = NULL;
 		switch (type) {
 		case GCRYSTAL:
 			ext = ".gcrystal";
@@ -233,31 +198,27 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 		case VRML:
 			ext = ".wrl";
 			break;
-		case JPEG:
-			ext = ".jpg";
-			break;
-		case PNG:
-			ext = ".png";
+		default:
 			break;
 		}
-		int i = strlen (filename) - strlen (ext);
-		if ((i > 0) && (!strcmp (filename +i, ext)))
-			filename2 = g_strdup (filename);
-		else
-			filename2 = g_strdup_printf ("%s%s", filename, ext);
-		GnomeVFSURI *uri = gnome_vfs_uri_new (filename2);
+		if (ext) { 
+			int i = strlen (filename) - strlen (ext);
+			if ((i <= 0) || (strcmp (filename +i, ext)))
+				filename2 += ext;
+		}
+		GnomeVFSURI *uri = gnome_vfs_uri_new (filename2.c_str ());
 		bool err = gnome_vfs_uri_exists (uri);
 		gnome_vfs_uri_unref (uri);
 		gint result = GTK_RESPONSE_YES;
 		if (err) {
-			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), filename2);
+			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), filename2.c_str ());
 			GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
 			gtk_window_set_icon_name (GTK_WINDOW (Box), "gcrystal");
 			result = gtk_dialog_run (Box);
 			gtk_widget_destroy (GTK_WIDGET (Box));
 			g_free (message);
 			if (result == GTK_RESPONSE_YES)
-				gnome_vfs_unlink (filename2);
+				gnome_vfs_unlink (filename2.c_str ());
 		}
 		map <string, string> options; // not used at the moment
 		if (result == GTK_RESPONSE_YES)
@@ -265,27 +226,36 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 			case GCRYSTAL:
 				Doc->SetFileName (filename2);
 				Doc->Save ();
+				GtkRecentData data;
+				data.display_name = (char*) Doc->GetTitle ();
+				data.description = NULL;
+				data.mime_type = "application/x-gcrystal";
+				data.app_name = "gcrystal";
+				data.app_exec = "gcrystal %u";
+				data.groups = NULL;
+				data.is_private =  FALSE;
+				gtk_recent_manager_add_full (GetRecentManager (), filename2.c_str (), &data);
+				Doc->RenameViews ();
 				break;
 			case VRML:
 				Doc->OnExportVRML (filename2);
 				break;
-			case JPEG:
-				Doc->SaveAsImage (filename2, "jpeg", options);
-				break;
-			case PNG:
-				Doc->SaveAsImage (filename2, "png", options);
+			case PIXBUF:
+				Doc->SaveAsImage (filename2, pixbuf_type, options);
 				break;
 			}
-		g_free (filename2);
 	} else {
 		if (strcmp (mime_type, "application/x-gcrystal"))
 			return true;
-		if (!Doc)
-			Doc = GetDoc (filename);
+		gcDocument *xDoc = GetDoc (filename);
+		if (xDoc)
+			Doc = xDoc;
+		else if (!pDoc->GetEmpty () || pDoc->GetDirty ())
+			Doc = NULL;
 		if (!Doc)
 			Doc = OnFileNew ();
 		if (Doc->GetFileName () && !strcmp (Doc->GetFileName(), filename)) {
-			if (!Doc->IsDirty ())
+			if (!Doc->GetDirty ())
 				return true;
 			else {
 				gchar* str = g_strdup_printf (_("\"%s\" has been modified since last saving. Do you wish to come back to saved version?"), Doc->GetTitle ());
@@ -296,57 +266,38 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 			}
 		}
 		if (Doc->Load (filename)) {
-			// TODO: change titles in every window
-	/*		gtk_label_set_text (pView->GetLabel (), pDoc->GetTitle ());
-			GtkLabel *pLabel = pView->GetMenuLabel ();
-			if (pLabel)
-				gtk_label_set_text (pLabel, pDoc->GetTitle ());*/
+			GtkRecentData data;
+			data.display_name = (char*) Doc->GetTitle ();
+			data.description = NULL;
+			data.mime_type = "application/x-gcrystal";
+			data.app_name = "gcrystal";
+			data.app_exec = "gcrystal %u";
+			data.groups = NULL;
+			data.is_private =  FALSE;
+			gtk_recent_manager_add_full (GetRecentManager (), filename, &data);
+			// change titles in every window and bring to front
+			list <CrystalView *> *Views = Doc->GetViews ();
+			list <CrystalView *>::iterator i, iend = Views->end ();
+			int n = 1, max = Views->size ();
+			char const *title = Doc->GetTitle ();
+			for (i = Views->begin (); i != iend; i++) {
+				gcWindow *window = dynamic_cast <gcView*> (*i)->GetWindow ();
+				GtkWindow *w = window->GetWindow ();
+				gtk_window_present (w);
+				if (max > 1) {
+					char *t = g_strdup_printf ("%s (%i)", title, n++);
+					gtk_window_set_title (w, t);
+					g_free (t);
+				} else
+					gtk_window_set_title (w, title);
+				window->ActivateActionWidget ("ui/MainMenu/FileMenu/Save", !Doc->GetReadOnly ());
+				window->ActivateActionWidget ("ui/MainToolbar/Save", !Doc->GetReadOnly ());
+			}
 		}
 	}
 	return false;
 }
 
-void gcApplication::OnBug ()
-{
-	if (!WebBrowser.size ())
-		return;
-	char *argv[3] = {NULL, PACKAGE_BUGREPORT, NULL};
-	argv[0] = (char*) WebBrowser.c_str();
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, NULL);
-}
-
-void gcApplication::ShowURI (string& uri)
-{
-	if (!WebBrowser.size ())
-		return;
-	char *argv[3] = {NULL, NULL, NULL};
-	argv[0] = (char*) WebBrowser.c_str();
-	argv[1] = (char*) uri.c_str ();
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, NULL);
-}
-
-void gcApplication::OnWeb ()
-{
-	if (!WebBrowser.size ())
-		return;
-	char *argv[3] = {NULL, "http://gchemutils.nongnu.org/", NULL};
-	argv[0] = (char*) WebBrowser.c_str();
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, NULL);
-}
-
-void gcApplication::OnMail ()
-{
-	if (!MailAgent.size ())
-		return;
-	char *argv[3] = {NULL, "mailto:gchemutils-main@nongnu.org", NULL};
-	argv[0] = (char*) MailAgent.c_str();
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, NULL);
-}
-
 void gcApplication::RemoveDocument (gcDocument *pDoc)
 {
 	m_Docs.remove (pDoc);
@@ -356,7 +307,7 @@ void gcApplication::RemoveDocument (gcDocument *pDoc)
 
 bool gcApplication::OnQuit ()
 {
-	while (m_Docs.size () > 1) {
+	while (m_Docs.size () > 0) {
 		m_pActiveDoc = m_Docs.front ();
 		if (!OnFileClose ())
 			return false;
