@@ -4,7 +4,7 @@
  * Gnome Crystal
  * document.cc 
  *
- * Copyright (C) 2000-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2000-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -93,17 +93,10 @@ gcDocument::gcDocument (gcApplication *pApp) :CrystalDoc (pApp)
 	Init ();
 	m_filename = NULL;
 	m_title = NULL;
-	m_bEmpty = true;
-	m_bDirty = false;
 	m_bClosing = false;
+	m_ReadOnly = false;
 }
 
-/*gcDocument::gcDocument(bool create_view)
-{
-	Init();
-	if (create_view) pView = m_pView = new gcView(this);
-}*/
-
 gcDocument::~gcDocument()
 {
 	if (m_filename != NULL) g_free(m_filename);
@@ -195,6 +188,10 @@ void gcDocument::SetCell(gcLattices lattice, gdouble a, gdouble b, gdouble c, gd
 
 void gcDocument::SetFileName (const string &filename)
 {
+	GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
+	gnome_vfs_get_file_info (filename.c_str (), info, GNOME_VFS_FILE_INFO_DEFAULT);
+	m_ReadOnly = !(info->permissions & (GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_GROUP_WRITE));
+	gnome_vfs_file_info_unref (info);
 	if (m_filename)
 		g_free (m_filename);
 	m_filename = g_strdup (filename.c_str ());
@@ -222,13 +219,6 @@ void gcDocument::SetTitle(const gchar* title)
 {
 	if (m_title) g_free(m_title);
 	m_title = g_strdup(title);
-	list<CrystalView*>::iterator view;
-	GtkLabel *label;
-	for (view = m_Views.begin(); view != m_Views.end(); view++)
-	{
-		label = ((gcView*)(*view))->GetLabel();
-		if (label) gtk_label_set_text(label, title);
-	}
 }
 
 void gcDocument::Save()
@@ -244,7 +234,8 @@ void gcDocument::Save()
 			Error (SAVE);
 			
 		xmlFreeDoc (xml);
-		m_bDirty = false;
+		SetDirty (false);
+		m_ReadOnly = false;	// if saving succeded, the file is not read only...
 	}
 	catch (int num) {
 		xmlFreeDoc (xml);
@@ -578,19 +569,22 @@ gcView *gcDocument::GetNewView()
 
 void gcDocument::AddView(gcView* pView)
 {
-	m_Views.push_back(pView);
-	if (!m_bEmpty) m_bDirty = true;
+	m_Views.push_back (pView);
+	RenameViews ();
+	if (!GetEmpty ())
+		SetDirty (true);
 }
 
 bool gcDocument::RemoveView (gcView* pView)
 {
 	if (m_Views.size () > 1) {
 		m_Views.remove (pView);
-		if (!m_bClosing && !m_bEmpty)
-			m_bDirty = true;
+		RenameViews ();
+		if (!m_bClosing && !GetEmpty ())
+			SetDirty (true);
 		return true;
 	}
-	if (IsDirty ()) {
+	if (GetDirty ()) {
 		if (!VerifySaved ())
 			return false;
 	}
@@ -610,7 +604,8 @@ void gcDocument::RemoveAllViews ()
 bool gcDocument::VerifySaved()
 {
 	m_bClosing = true;
-	if (!m_bDirty) return true;
+	if (!GetDirty ())
+		return true;
 	gchar* str = g_strdup_printf(_("\"%s\" has been modified.  Do you wish to save it?"), m_title);
 	GtkWidget* mbox;
 	int res;
@@ -622,7 +617,7 @@ bool gcDocument::VerifySaved()
 		gtk_widget_destroy(mbox);
 		if (res == GTK_RESPONSE_YES) {
 			if (m_filename == NULL) {
-				list<char const*> l;
+				list<string> l;
 				l.push_front ("application/x-gcrystal");
 				FileChooser (m_App, true, l, this);
 			}
@@ -631,22 +626,13 @@ bool gcDocument::VerifySaved()
 		}
 	}
 	while ((res == GTK_RESPONSE_YES) && (m_filename == NULL));
-	if (res == GTK_RESPONSE_NO) m_bDirty = false;
+	if (res == GTK_RESPONSE_NO)
+		SetDirty (false);
 	else if (res == GTK_RESPONSE_CANCEL) m_bClosing = false;
 	g_free(str);
 	return (res != GTK_RESPONSE_CANCEL);
 }
 
-void gcDocument::NotifyDialog (Dialog* dialog)
-{
-	m_Dialogs.push_front(dialog);
-}
-
-void gcDocument::RemoveDialog (Dialog* dialog)
-{
-	m_Dialogs.remove(dialog);
-}
-
 CrystalView* gcDocument::CreateNewView()
 {
 	return new gcView(this);
@@ -674,7 +660,7 @@ const char* gcDocument::GetProgramId()
 
 void gcDocument::SaveAsImage (const string &filename, char const *type, map<string, string>& options)
 {
-	m_pActiveView->SaveAsImage (filename, type, options, GetApp ()->GetImageResolution ());
+	m_pActiveView->SaveAsImage (filename, type, options, GetApp ()->GetImageWidth (), GetApp ()->GetImageHeight ());
 }
 
 bool gcDocument::LoadNewView (xmlNodePtr node)
@@ -686,3 +672,23 @@ bool gcDocument::LoadNewView (xmlNodePtr node)
 		delete pWindow;
 	return result;
 }
+
+void gcDocument::RenameViews ()
+{
+	list <CrystalView *>::iterator i, iend = m_Views.end ();
+	int n = 1, max = m_Views.size ();
+	for (i = m_Views.begin (); i != iend; i++) {
+		gcWindow *window = dynamic_cast <gcView*> (*i)->GetWindow ();
+		GtkWindow *w = window->GetWindow ();
+		if (!w)
+			continue;
+		if (max > 1) {
+			char *t = g_strdup_printf ("%s (%i)", m_title, n++);
+			gtk_window_set_title (w, t);
+			g_free (t);
+		} else
+			gtk_window_set_title (w, m_title);
+		window->ActivateActionWidget ("ui/MainMenu/FileMenu/Save", !m_ReadOnly);
+		window->ActivateActionWidget ("ui/MainToolbar/Save", !m_ReadOnly);
+	}
+}
